var messageEditor = Class.create();

messageEditor.prototype = {
	openTags: new Array(),
	toolbarHeight: 0,
	currentTheme: '',
	themePath: '',
	openDropDownMenu: null,

	setTheme: function(theme)
	{
		if(this.currentTheme != '' || $('editorTheme')) {
			$('editorTheme').remove();
		}

		var stylesheet = document.createElement('link');
		stylesheet.setAttribute('rel', 'stylesheet');
		stylesheet.setAttribute('type', 'text/css');
		stylesheet.setAttribute('href', this.baseURL + 'editor_themes/'+theme+'/stylesheet.css');
		document.getElementsByTagName('head')[0].appendChild(stylesheet);
		this.currentTheme = theme;
		this.themePath = this.baseURL + 'editor_themes/'+theme;
	},

	initialize: function(textarea, options)
	{
		// Sorry Konqueror, but due to a browser bug out of control with textarea values
		// you do not get to use the fancy editor.

		if(MyBB.browser == "konqueror" || (typeof(mybb_editor_disabled) != "undefined" && mybb_editor_disabled == true))
		{
			return false;
		}

		// Establish the base path to this javascript file
		$$('script').each(function(script) {
			if(script.src && script.src.indexOf('editor.js') != -1) {
				this.baseURL = script.src.replace(/editor\.js(.*?)$/, '');
			}
		}, this);

		this.options = options;

		if(this.options)
		{
			if(!this.options.lang)
			{
				return false;
			}

			if(!this.options.rtl)
			{
				this.options.rtl = 0;
			}
		}

		if(this.options && this.options.theme)
		{
			this.setTheme(this.options.theme);
		}
		else
		{
			this.setTheme('default');
		}

		// Defines an array of fonts to be shown in the font drop down.
		this.fonts = new Object();
		this.fonts["Arial"] = "Arial";
		this.fonts["Courier"] = "Courier";
		this.fonts["Impact"] = "Impact";
		this.fonts["Tahoma"] = "Tahoma";
		this.fonts["Times New Roman"] = "Times New Roman";
		this.fonts["Trebuchet MS"] = "Trebuchet MS";
		this.fonts["Verdana"] = "Verdana";

		// An array of font sizes to be shown.
		this.sizes = new Object();
		this.sizes["xx_small"] = this.options.lang.size_xx_small;
		this.sizes["x_small"] = this.options.lang.size_x_small;
		this.sizes["small"] = this.options.lang.size_small;
		this.sizes["medium"] = this.options.lang.size_medium;
		this.sizes["large"] = this.options.lang.size_large;
		this.sizes["x_large"] = this.options.lang.size_x_large;
		this.sizes["xx_large"] = this.options.lang.size_xx_large;


		// An array of colours to be shown.
		this.colors = new Object();
		this.colors[1] = "#800000";
		this.colors[2] = "#8B4513";
		this.colors[3] = "#006400";
		this.colors[4] = "#2F4F4F";
		this.colors[5] = "#000080";
		this.colors[6] = "#4B0082";
		this.colors[7] = "#800080";
		this.colors[8] = "#000000";
		this.colors[9] = "#FF0000";
		this.colors[10] = "#DAA520";
		this.colors[11] = "#6B8E23";
		this.colors[12] = "#708090";
		this.colors[13] = "#0000CD";
		this.colors[14] = "#483D8B";
		this.colors[15] = "#C71585";
		this.colors[16] = "#696969";
		this.colors[17] = "#FF4500";
		this.colors[18] = "#FFA500";
		this.colors[19] = "#808000";
		this.colors[20] = "#4682B4";
		this.colors[21] = "#1E90FF";
		this.colors[22] = "#9400D3";
		this.colors[23] = "#FF1493";
		this.colors[24] = "#A9A9A9";
		this.colors[25] = "#FF6347";
		this.colors[26] = "#FFD700";
		this.colors[27] = "#32CD32";
		this.colors[28] = "#87CEEB";
		this.colors[29] = "#00BFFF";
		this.colors[30] = "#9370DB";
		this.colors[31] = "#FF69B4";
		this.colors[32] = "#DCDCDC";
		this.colors[33] = "#FFDAB9";
		this.colors[34] = "#FFFFE0";
		this.colors[35] = "#98FB98";
		this.colors[36] = "#E0FFFF";
		this.colors[37] = "#87CEFA";
		this.colors[38] = "#E6E6FA";
		this.colors[39] = "#DDA0DD";
		this.colors[40] = "#FFFFFF";
		
		// An array of video services to be shown (youtube, vimeo, etc) 
		this.videos = new Object();
		this.videos["dailymotion"] = this.options.lang.video_dailymotion;
		this.videos["googlevideo"] = this.options.lang.video_googlevideo;
		this.videos["metacafe"] = this.options.lang.video_metacafe;
		this.videos["myspacetv"] = this.options.lang.video_myspacetv;
		this.videos["vimeo"] = this.options.lang.video_vimeo;
		this.videos["yahoo"] = this.options.lang.video_yahoo;
		this.videos["youtube"] = this.options.lang.video_youtube;

		// Here we get the ID of the textarea we're replacing and store it.
		this.textarea = textarea;

		// Only swap it over once the page has loaded (add event)
		if(MyBB.page_loaded == 1)
		{
			this.showEditor();
		}
		else
		{
			Event.observe(document, "dom:loaded", this.showEditor.bindAsEventListener(this));
		}
	},

	showEditor: function()
	{
		// Assign the old textarea to a variable for later use.
		oldTextarea = $(this.textarea);

		// Now this.textarea becomes the new textarea ID
		this.textarea += "_new";

		// Begin the creation of our new editor.

		this.editor = document.createElement("div");
		this.editor.style.position = "relative";
		this.editor.style.display = "none";
		this.editor.className = "messageEditor";

		// Append the new editor
		oldTextarea.parentNode.insertBefore(this.editor, oldTextarea);

		// Determine the overall height and width - messy, but works
		w = oldTextarea.getDimensions().width+"px";
		if(!w || parseInt(w) < 400)
		{
			w = "400px";
		}
		if(this.options && this.options.height)
		{
			h = this.options.height;
		}
		else if(oldTextarea.offsetHeight)
		{
			h = oldTextarea.offsetHeight+"px";
		}
		else if(oldTextarea.clientHeight)
		{
			h = oldTextarea.clientHeight+"px";
		}
		else if(oldTextarea.style.height)
		{
			h = oldTextarea.style.height;
		}
		else
		{
			h = "400px";
		}
		this.editor.style.width = w;
		this.editor.style.height = h;

		this.createToolbarContainer('top');

		this.createToolbar('toggle', {
			container: 'top',
			alignment: 'right',
			items: [
				{type: 'button', name: 'removeformat', sprite: 'removeformat', insert: 'removeformat', extra: 1, title: this.options.lang.removeformat, exec:'removeformat'},
				{type: 'button', name: 'toggle', sprite: 'toggle', insert: 'toggle', title: this.options.lang.toggle}
			]
		});
		this.createToolbar('resize', {
			container: 'top',
			alignment: 'right',
			items: [
				{type: 'button', name: 'resize1', sprite: 'resize1', insert: 'resize1', title: this.options.lang.resize1},
				{type: 'button', name: 'resize-1', sprite: 'resize-1', insert: 'resize-1', title: this.options.lang.resize0}
			]
		});
		this.createToolbar('closetags', {
			container: 'top',
			alignment: 'right',
			items: [
				{type: 'button', name: 'close_tags', insert: 'zzzz', sprite: 'close_tags', width: 80, style: {visibility: 'hidden'}}
			]
		});
		this.createToolbar('topformatting', {
			container: 'top',
			items: [
				{type: 'dropdown', name: 'font', insert: 'font', title: this.options.lang.font, options: this.fonts, exec:'fontname'},
				{type: 'dropdown', name: 'size', insert: 'size', title: this.options.lang.size, options: this.sizes, exec:'fontsize'},
				{type: 'button', name: 'color', insert: 'color', dropdown: true, color_select: true, image: 'color.gif', draw_option: this.drawColorOption, options: this.colors, exec:'foreColor'}
			]
		});

		this.createToolbarContainer('bottom');

		this.createToolbar('insertables', {
			container: 'bottom',
			alignment: 'right',
			items: [
				{type: 'button', name: 'list_num', sprite: 'list_num', insert: 'list', extra: 1, title: this.options.lang.title_numlist, exec:'insertorderedlist'},
				{type: 'button', name: 'list_bullet', sprite: 'list_bullet', insert: 'list', title: this.options.lang.title_bulletlist, exec:'insertunorderedlist'},
				{type: 'separator'},
				{type: 'button', name: 'img', sprite: 'image', insert: 'image', extra: 1, title: this.options.lang.title_image},
				{type: 'button', name: 'url', sprite: 'link', insert: 'url', title: this.options.lang.title_hyperlink},
				{type: 'button', name: 'email', sprite: 'email', insert: 'email', extra: 1, title: this.options.lang.title_email},
				{type: 'button', name: 'unlink', sprite: 'unlink', insert: 'unlink', extra: 1, title: this.options.lang.unlink, exec:'unlink'},
				{type: 'separator'},
				{type: 'button', name: 'quote', sprite: 'quote', insert: 'quote', title: this.options.lang.title_quote},
				{type: 'button', name: 'code', sprite: 'code', insert: 'code', title: this.options.lang.title_code},
				{type: 'button', name: 'php', sprite: 'php', insert: 'php', title: this.options.lang.title_php},
				{type: 'button', name: 'video', insert: 'video', image: 'television.gif', dropdown: true, title: this.options.lang.title_video, options: this.videos}
			]
		});
		this.createToolbar('formatting', {
			container: 'bottom',
			items: [
				{type: 'button', name: 'b', sprite: 'bold', insert: 'b', title: this.options.lang.title_bold, exec:'bold'},
				{type: 'button', name: 'i', sprite: 'italic', insert: 'i', title: this.options.lang.title_italic, exec:'italic'},
				{type: 'button', name: 'u', sprite: 'underline', insert: 'u', title: this.options.lang.title_underline, exec:'underline'},
				{type: 'separator'},
				{type: 'button', name: 'align_left', sprite: 'align_left', insert: 'align', extra: 'left', title: this.options.lang.title_left, exec:'justifyleft'},
				{type: 'button', name: 'align_center', sprite: 'align_center', insert: 'align', extra: 'center', title: this.options.lang.title_center, exec:'justifycenter'},
				{type: 'button', name: 'align_right', sprite: 'align_right', insert: 'align', extra: 'right', title: this.options.lang.title_right, exec:'justifyright'},
				{type: 'button', name: 'align_justify', sprite: 'align_justify', insert: 'align', extra: 'justify', title: this.options.lang.title_justify, exec:'justifyfull'}
			]
		});

		// Create our new text area
		areaContainer = document.createElement("div");
		areaContainer.style.clear = "both";

		// Set the width/height of the area
		subtract = 20;
		subtract2 = 12;
		areaContainer.style.height = parseInt(Element.getDimensions(this.editor).height)-this.toolbarHeight-subtract+"px";
		areaContainer.style.width = parseInt(Element.getDimensions(this.editor).width)-subtract2+"px";

		// Create text area
		textInput = document.createElement("textarea");
		textInput.setAttribute("cols", oldTextarea.getAttribute("cols"));
		textInput.setAttribute("rows", oldTextarea.getAttribute("rows"));
		textInput.id = this.textarea;
		textInput.name = oldTextarea.name+"_new";
		textInput.style.height = parseInt(areaContainer.style.height)+"px";
		textInput.style.width = parseInt(areaContainer.style.width)+"px";


		wysiwygInput = document.createElement("iframe");
		wysiwygInput.style.border = "1px solid #0f5c8e";

		wysiwygInput.id = "wysiwyg_"+this.textarea;
		wysiwygInput.style.height = parseInt(areaContainer.style.height)+"px";
		wysiwygInput.style.width = parseInt(areaContainer.style.width)+"px";

	
		
		if(oldTextarea.value != '')
		{
			textInput.value = oldTextarea.value;
		}

		if(oldTextarea.tabIndex)
		{
			textInput.tabIndex = oldTextarea.tabIndex;
		}

		areaContainer.appendChild(wysiwygInput);
		this.editor.appendChild(areaContainer);

		if(oldTextarea.form)
		{
			Event.observe(oldTextarea.form, "submit", this.closeTags.bindAsEventListener(this));
			Event.observe(oldTextarea.form, "submit", this.updateOldArea.bindAsEventListener(this));
		}


		// Hide the old editor
		oldTextarea.style.visibility = "hidden";
		oldTextarea.style.position = "absolute";
		oldTextarea.style.top = "-1000px";
		oldTextarea.id += "_old";
		this.oldTextarea = oldTextarea;
		
		var wysiwyg_rtl;
		if(this.options.rtl){
		wysiwyg_rtl=' dir="rtl"';
		}
	
		this.wysiwyg = document.getElementsByTagName('iframe')[0];		
		this.e=this.wysiwyg.contentWindow.document; this.e.open();
		var m='<html'+wysiwyg_rtl+'><head id="'+wysiwygInput.id+'_head"></head><body id="'+wysiwygInput.id+'_body" style="font-family:tahoma; font-size:12px;">'+(BBcodetoHTML(textInput.value)||oldTextarea.value)+'</body></html>';
		this.e.write(m);
		this.e.close(); this.e.designMode='on';
		if(MyBB.browser != 'ie')
		{
		this.e.body.style.background = "#ffffff";
		}
	areaContainer.appendChild(textInput);
			document.getElementById(this.textarea).style.display="none";
		this.d = 1;

		this.editor.style.display = "";
		if(!this.d){
		Event.observe(textInput, "keyup", this.updateOldArea.bindAsEventListener(this));

		if(MyBB.browser == 'ie') {
			Event.observe($(this.textarea), 'focus', function() {
				this.trackingCaret = true;
			}.bindAsEventListener(this));
			Event.observe($(this.textarea), 'blur', function() {
				this.trackingCaret = false;
			}.bindAsEventListener(this));
			Event.observe($(this.textarea), 'mousedown', function() {
				this.trackingCaret = true;
				this.storeCaret();
			}.bindAsEventListener(this));
		}

		Event.observe(textInput, "blur", this.updateOldArea.bindAsEventListener(this));
		}
			
Event.observe(oldTextarea.form, "submit", this.posting.bindAsEventListener(this));
	},
	posting : function()
	{
		if(this.d){
		areaContainer.appendChild(textInput);
		textInput.style.display="none";
		textInput.value = HTMLtoBBcode(this.e.body.innerHTML) ;
		this.oldTextarea.value = textInput.value;
		if(oldTextarea.form.id=="quick_reply_form")	{
		this.e.body.innerHTML="";
		}
		}
	},
	toggle : function()
	{
		if(!this.d){
		document.getElementById(this.textarea).style.display="none";
		document.getElementsByTagName('iframe')[0].style.display="";
		this.e.body.innerHTML = BBcodetoHTML(textInput.value) ;
		this.d=1;
		}
		else{
		document.getElementsByTagName('iframe')[0].style.display="none";
		if(!document.getElementById(this.textarea)){
		document.getElementsByTagName('textarea')[0].style.display="";
		}
		else{
		document.getElementById(this.textarea).style.display="";
		}
		areaContainer.appendChild(textInput);
		if(MyBB.browser == 'ie') {
			Event.observe($(this.textarea), 'focus', function() {
				this.trackingCaret = true;
			}.bindAsEventListener(this));
			Event.observe($(this.textarea), 'blur', function() {
				this.trackingCaret = false;
			}.bindAsEventListener(this));
			Event.observe($(this.textarea), 'mousedown', function() {
				this.trackingCaret = true;
				this.storeCaret();
			}.bindAsEventListener(this));
		}
		Event.observe(textInput, "blur", this.updateOldArea.bindAsEventListener(this));
	
		textInput.value = HTMLtoBBcode(this.e.body.innerHTML) ;
		this.d=0;
		
				if(MyBB.browser != 'ie')
		{
		document.getElementById(this.textarea).focus();
		}
else{
		var range = $(this.textarea).createTextRange();
		range.collapse(true);
		range.moveStart('character', textInput.value.length);
		range.moveEnd('character', textInput.value.length);
		range.select();
		}
		}
	},

	drawColorOption: function(option)
	{
		var item = document.createElement('li');
		item.extra = option.value;
		item.className = 'editor_dropdown_color_item';
		item.innerHTML = '<a style="background-color: '+option.value+'"></a>';
		return item;
	},

	createToolbarContainer: function(name)
	{
		if($('editor_toolbar_container_'+name)) return;

		var container = document.createElement("div");
		container.id = 'editor_toolbar_container_'+name;
		container.className = 'toolbar_container';

		this.editor.appendChild(container);

		this.toolbarHeight += 28;

		return container;
	},

	createToolbar: function(name, options)
	{
		if(typeof(options.container) == 'undefined')
		{
			options.container = this.createToolbarContainer('auto_'+name);
		}
		else {
			options.container = $('editor_toolbar_container_'+options.container);
			if(!options.container) return;
		}

		if($('editor_toolbar_'+name)) return;

		var toolbar = document.createElement('div');
		toolbar.id = 'editor_toolbar_'+name;
		toolbar.className = 'toolbar';

		var clear = document.createElement('br');
		clear.style.clear = 'both';
		toolbar.appendChild(clear);

		if(options.alignment && options.alignment == 'right') {
			toolbar.className += ' float_right';
		}
		options.container.appendChild(toolbar);
		if(typeof(options.items) == 'object') {
			for(var i = 0; i < options.items.length; ++i) {
				this.addToolbarItem(toolbar, options.items[i]);
			}
		}
		// add closing item
		if(toolbar.lastChild.previousSibling)
			toolbar.lastChild.previousSibling.className += ' toolbar_button_group_last';
	},
	
	setElementState: function(element, state) {
		element.addClassName('toolbar_'+state);
		
		if(element.hasClassName('toolbar_button_group_first')) {
			if(state == 'clicked') {
				append = 'toolbar_clicked';
			}
			else if(state == 'hover') {
				append = 'toolbar_hover';
			}
			append += '_button_group_first';
			element.addClassName(append);
		}
		
		if(element.hasClassName('toolbar_button_group_last')) {
			if(state == 'clicked') {
				append = 'toolbar_clicked';
			}
			else if(state == 'hover') {
				append = 'toolbar_hover';
			}
			append += '_button_group_last';
			element.addClassName(append);
		}
	},
	
	removeElementState: function(element, state)
	{
		element.removeClassName('toolbar_'+state);
		
		if(element.hasClassName('toolbar_button_group_first')) {
			if(state == 'clicked') {
				append = 'toolbar_clicked';
			}
			else if(state == 'hover') {
				append = 'toolbar_hover';
			}
			append += '_button_group_first';
			element.removeClassName(append);
		}
		
		if(element.hasClassName('toolbar_button_group_last')) {
			if(state == 'clicked') {
				append = 'toolbar_clicked';
			}
			else if(state == 'hover') {
				append = 'toolbar_hover';
			}
			append += '_button_group_last';
			element.removeClassName(append);
		}	
	},

	dropDownMenuItemClick: function(e)
	{
		this.restartEditorSelection();
		element = Event.element(e);

		if(!element)
			return;
		
		if(!element.extra)
			element = element.up('li');
		
		var mnu = element.up('ul');
		var dropdown = this.getElementToolbarItem(mnu);
		var label = dropdown.down('.editor_dropdown_label');

		if(!dropdown.insertText || (mnu.activeItem && mnu.activeItem == element))
			return;
		
		mnu.lastItemValue = element.extra;
		if(!this.d)
		{
		this.insertMyCode(dropdown.insertText, element.extra, element.wysiwygText);

		if(this.getSelectedText($(this.textarea)))
		{
			this.setDropDownMenuActiveItem(dropdown, 0);
		}
		else
		{
			if(label)
			{
				label.innerHTML = element.innerHTML;
				label.style.overflow = 'hidden';
			}
			var sel_color = dropdown.down('.editor_button_color_selected')
			if(sel_color)
			{
				sel_color.style.backgroundColor = element.extra;
				var use_default = dropdown.down('.editor_dropdown_color_item_default');
				if(use_default) use_default.style.display = '';
			}
			mnu.activeItem = element;
			element.addClassName('editor_dropdown_menu_item_active');
		}
		}
		else{
			 this.wysiwyg.contentWindow.focus();
	 if(dropdown.wysiwygText=="fontsize"){
	 if(element.extra=="xx_small")element.extra="0";
	 else if(element.extra=="x_small")element.extra="1";
	 else if(element.extra=="small")element.extra="2";
	 else if(element.extra=="medium")element.extra="3";
	 else if(element.extra=="large")element.extra="4";
	 else if(element.extra=="x_large")element.extra="5";
	 else if(element.extra=="xx_large")element.extra="6";
	 }
this.e.execCommand(dropdown.wysiwygText, false, element.extra);

		}
		this.hideOpenDropDownMenu();
		Event.stop(e);
	},

	setDropDownMenuActiveItem: function(element, index)
	{
		if(element == null)
		{
			return;
		}
		var mnu = element.down('ul');
		var label = element.down('.editor_dropdown_label');

		if(mnu.activeItem)
		{
			mnu.activeItem.removeClassName('editor_dropdown_menu_item_active');
			mnu.activeItem = null;
		}

		if(index > 0)
		{
			var item = mnu.childNodes[index];
			if(!item) return;
			mnu.activeItem = item;
			if(label)
			{
				label.innerHTML = item.innerHTML;
			}

			var sel_color = element.down('.editor_dropdown_color_selected')
			if(sel_color)
			{
				sel_color.style.backgroundColor = item.style.backgroundColor;
				mnu.lastItemValue = item.insertExtra;
				var use_default = element.down('.editor_dropdown_color_item_default');
				if(use_default) use_default.style.display = '';
			}
			item.addClassName('editor_dropdown_menu_item_active');
		}
		else
		{
			if(label)
			{
				label.innerHTML = mnu.childNodes[0].innerHTML;
			}

			var sel_color = element.down('.editor_button_color_selected')
			if(sel_color)
			{
				//sel_color.style.backgroundColor = '';
				var use_default = element.down('.editor_dropdown_color_item_default');
				if(use_default) use_default.style.display = 'none';
			}
			this.removeElementState(element, 'clicked');
		}
	},

	createDropDownMenu: function(options)
	{
		var dropdown = document.createElement('div');
		dropdown.itemType = options.type;
		if(options.image || options.sprite)
			dropdown.className = 'toolbar_dropdown_image';
		else
			dropdown.className = 'toolbar_dropdown';
		dropdown.wysiwygText = options.exec;
		dropdown.className += ' editor_dropdown toolbar_dropdown_'+options.name;
		dropdown.id = 'editor_item_'+options.name;

		Event.observe(dropdown, 'mouseover', this.dropdownmouseover.bindAsEventListener(this));
		Event.observe(dropdown, 'mouseout', this.dropdownmouseout.bindAsEventListener(this));
		dropdown.insertText = options.insert;

		// create the dropdown label container
		var label = document.createElement('div');
		label.className = 'editor_dropdown_label';
		if(options.title)
		{
			label.innerHTML = options.title;
		}
		else
		{
			label.innerHTML = '&nbsp;';
		}
		dropdown.appendChild(label)

		// create the arrow
		var arrow = document.createElement('div');
		arrow.className = 'editor_dropdown_arrow';
		dropdown.appendChild(arrow);

		// create the menu item container
		var mnu = this.buildDropDownMenu(options);

		Event.observe(dropdown, 'click', this.toggleDropDownMenu.bindAsEventListener(this));
		dropdown.appendChild(mnu);
		return dropdown;
	},
dropdownmouseover: function(e)
		{
				element = Event.element(e);
		if(MyBB.browser == 'ie') {
		if(this.d) {
		element.unselectable='on';
		}
		}
		if(!element)
			return;
		if(!element.itemType)
			element = this.getElementToolbarItem(element);
			
			
			
			this.storeCaret();
			element.addClassName('toolbar_dropdown_over');
		},
		dropdownmouseout: function(e)
		{
				element = Event.element(e);
		if(MyBB.browser == 'ie') {
		if(this.d) {
		element.unselectable='on';
		}
		}
		if(!element)
			return;
		if(!element.itemType)
			element = this.getElementToolbarItem(element);
			
			
			
			this.storeCaret();
			element.removeClassName('toolbar_dropdown_over');
		},
	buildDropDownMenu: function(options)
	{
		var mnu = document.createElement('ul');
		mnu.className = 'editor_dropdown_menu';
		mnu.style.display = 'none';

		// create the first item
		if(options.title)
		{
			var item = document.createElement('li');
			item.className = 'editor_dropdown_menu_title';
			item.innerHTML = options.title;
			mnu.appendChild(item);
			Event.observe(item, 'click', function()
			{
				if(mnu.activeItem)
				{
					this.restartEditorSelection();
					this.insertMyCode(dropdown.insertText, '-');
				}
				this.setDropDownMenuActiveItem(dropdown, 0);
			}.bindAsEventListener(this));
		}
		
		$H(options.options).each(function(option)
		{
			if(options.draw_option)
			{
				item = options.draw_option(option)
			}
			else
			{
				var item = document.createElement('li');
				item.innerHTML = option.value;

				var content = document.createElement('span');
				item.appendChild(content);
				item.extra = option.key;
			}
			if(MyBB.browser == 'ie') {
			if(this.d) {
			item.unselectable='on';
			}
			}
			Event.observe(item, 'click', this.dropDownMenuItemClick.bindAsEventListener(this));
			Event.observe(item, 'mouseover', function()
			{
				item.addClassName('editor_dropdown_menu_item_over');
			});
			Event.observe(item, 'mouseout', function()
			{
				item.removeClassName('editor_dropdown_menu_item_over');
			});
			mnu.appendChild(item);
		}, this);
		return mnu;
	},

	toggleDropDownMenu: function(e)
	{
		element = Event.element(e);
		if(MyBB.browser == 'ie') {
		if(this.d) {
		element.unselectable='on';
		}
		}
		if(!element)
			return;
		if(!element.itemType)
			element = this.getElementToolbarItem(element);
		
		var mnu = $(element).down('ul');
		
		// This menu is already open, close it
		if(mnu.style.display != 'none')
		{
			mnu.style.display = 'none';
			element.removeClassName('editor_dropdown_menu_open');
			this.removeElementState(element, 'clicked');
			this.openDropDownMenu = null;
			Event.stopObserving(document, 'click', this.hideOpenDropDownMenu.bindAsEventListener(this));
		}
		// Opening this menu
		else
		{
			// If a menu is already open, close it first
			this.showDropDownMenu(mnu);
		}
		this.removeElementState(element, 'clicked');
		Event.stop(e);
	},

	showDropDownMenu: function(mnu)
	{
		this.hideOpenDropDownMenu();
		mnu.style.display = '';
		element = this.getElementToolbarItem(mnu);
		element.addClassName('editor_dropdown_menu_open');
		this.setElementState(element, 'clicked');
		this.openDropDownMenu = mnu;
		Event.observe(document, 'click', this.hideOpenDropDownMenu.bindAsEventListener(this));
	},

	hideOpenDropDownMenu: function()
	{
		if(!this.openDropDownMenu) return;
		this.openDropDownMenu.style.display = 'none';
		this.getElementToolbarItem(this.openDropDownMenu).removeClassName('editor_dropdown_menu_open');
		var dropDown = this.getElementToolbarItem(this.openDropDownMenu);
		this.removeElementState(element, 'clicked');
		this.openDropDownMenu = null;
		Event.stopObserving(document, 'click', this.hideOpenDropDownMenu.bindAsEventListener(this));
	},

	getElementToolbarItem: function(elem)
	{
		var parent = elem;
		do {
			if(parent.insertText) return parent;
			parent = parent.parentNode;
		} while($(parent));

		return false;
	},

	storeCaret: function()
	{
		if(MyBB.browser != 'ie' || !this.trackingCaret)
		{
			return;
		}
		
		// Internet explorer errors if you try and select an element... so just handle that by try catch
		try {
			var range = document.selection.createRange();
			var range_all = document.body.createTextRange();
			range_all.moveToElementText($(this.textarea));
			for(var sel_start = 0; range_all.compareEndPoints('StartToStart', range) < 0; sel_start++)
				range_all.moveStart('character', 1);

			var range_all = document.body.createTextRange();
			range_all.moveToElementText($(this.textarea));
			for(var sel_end = 0; range_all.compareEndPoints('StartToEnd', range) < 0; sel_end++)
				range_all.moveStart('character', 1);

			this.lastCaretS = sel_start;
			this.lastCaretE = sel_end;
		} catch(e) { }
	},

	restartEditorSelection: function()
	{
	if(!this.d){
		if(MyBB.browser != 'ie')
		{
			return;
		}

		var range = $(this.textarea).createTextRange();
		range.collapse(true);
		range.moveStart('character', this.lastCaretS);
		range.moveEnd('character', this.lastCaretE - this.lastCaretS);
		range.select();
		}
	},

	addToolbarItem: function(toolbar, options)
	{
		if(typeof(toolbar) == 'string')
		{
			toolbar = $('editor_toolbar_'+toolbar);
		}

		if(!$(toolbar)) return;

		// Does this item already exist?
		if($('editor_item_'+options.name)) return;

		insert_first_class = false;

		// Is this the first item? childnodes = 1 (closing br) or lastchild.previousSibling = sep
		if(toolbar.childNodes.length == 1 || (toolbar.lastChild.previousSibling && toolbar.lastChild.previousSibling.className.indexOf('toolbar_sep') > -1 || (toolbar.lastChild.previousSibling.className.indexOf('editor_dropdown') > -1 && options.type != 'dropdown')))
		{
			insert_first_class = true;
		}

		if(options.type == "dropdown")
		{
			var dropdown = this.createDropDownMenu(options);
			if(dropdown)
				toolbar.insertBefore(dropdown, toolbar.lastChild);

			if(insert_first_class == true)
				dropdown.className += ' toolbar_dropdown_group_first';
		}
		else if(options.type == 'button')
		{
			var button = this.createToolbarButton(options)
			toolbar.insertBefore(button, toolbar.lastChild);

			if(insert_first_class == true)
				button.className += ' toolbar_button_group_first';
		}
		else if(options.type == 'separator')
		{
			if(toolbar.lastChild.previousSibling && !$(toolbar.lastChild.previousSibling).hasClassName('toolbar_dropdown'))
			{
				toolbar.lastChild.previousSibling.className += ' toolbar_button_group_last';
			}
			var separator = document.createElement("span");
			separator.itemType = options.type;
			separator.className = "toolbar_sep";
			toolbar.insertBefore(separator, toolbar.lastChild);
		}
	},

	createToolbarButton: function(options)
	{
		var button = document.createElement('span');
		button.itemType = options.type;
		button.id = 'editor_item_'+options.name;
		if(typeof(options.title) != 'undefined')
		{
			button.title = options.title;
		}
		button.className = 'toolbar_button toolbar_normal toolbar_button_'+options.name;

		if(typeof(options.style) == 'object')
		{
			$H(options.style).each(function(item) {
				eval('button.style.'+item.key+' = "'+item.value+'";');
			});
		}
		button.insertText = options.insert;
		button.wysiwygText = options.exec;
		button.insertExtra = '';
		if(typeof(options.extra) != 'undefined')
			button.insertExtra = options.extra;
		
		if(typeof(options.sprite) != 'undefined')
		{
			var img = document.createElement('span');
			img.className = 'toolbar_sprite toolbar_sprite_'+options.sprite;
		}
		else
		{
			var img = document.createElement('img');
			img.src = this.themePath + "/images/" + options.image;
		}
		button.appendChild(img);

		if(options.dropdown)
		{
			if(options.color_select == true)
			{
				var sel = document.createElement('em');
				sel.className = 'editor_button_color_selected';
				button.appendChild(sel);
			}
			// create the arrow
			var arrow = document.createElement('u');
			arrow.className = 'toolbar_button_arrow';
			button.appendChild(arrow);
			button.className += ' toolbar_button_with_arrow';
		}

		var end = document.createElement('strong');
		button.appendChild(end);

		// Create the actual drop down menu
		if(options.dropdown)
		{
			// create the menu item container
			var mnu = this.buildDropDownMenu(options);

			Event.observe(arrow, 'click', this.toggleDropDownMenu.bindAsEventListener(this));
			Event.observe(button, 'click', this.toggleDropDownMenu.bindAsEventListener(this));
			Event.observe(arrow, 'mouseover', function(e)
			{
				elem = Event.element(e);
				if(!elem) return;
				elem.parentNode.addClassName('toolbar_button_over_arrow');
			});
			Event.observe(arrow, 'mouseout', function(e)
			{
				elem = Event.element(e);
				if(!elem) return;
				elem.parentNode.removeClassName('toolbar_button_over_arrow');
			});
			button.appendChild(mnu);
			button.dropdown = true;
			button.menu = mnu;
		}

		// Does this button have enabled/disabled states?
		if(options.disabled_img || options.disabled_sprite)
		{
			button.disable = function()
			{
				if(button.disabled == true) return;

				if(options.disabled_sprite)
				{
					img.removeClassName('toolbar_sprite_'+options.sprite);
					img.addClassName('toolbar_sprite_disabled_'+options.disabled_sprite);
				}
				else
					img.src = this.themePath + '/images/' + options.disabled_img;

				button.disabled = true;
			};

			button.enable = function()
			{
				if(!button.disabled) return;

				if(options.disabled_sprite)
				{
					img.removeClassName('toolbar_sprite_disabled_'+options.disabled_sprite);
					img.addClassName('toolbar_sprite_'+options.sprite);
				}
				else
					img.src = this.themePath + '/images/' + options.image;

				button.enabled = true;
			};

			if(options.disabled && options.disabled == true)
			{
				button.disable();
				button.disabled = true;
			}
			else
				button.disabled = false;
		}

		Event.observe(button, "mouseover", this.toolbarItemHover.bindAsEventListener(this));
		Event.observe(button, "mouseout", this.toolbarItemOut.bindAsEventListener(this));

		if(!options.dropdown)
		{
			// Dropdown event listener is above...
			Event.observe(button, "click", this.toolbarItemClick.bindAsEventListener(this));
			
		}
		return button;
	},

	updateOldArea: function(e)
	{
		this.oldTextarea.value = $(this.textarea).value;
	},

	toolbarItemOut: function(e)
	{
		this.storeCaret();
		element = Event.element(e);
if(MyBB.browser == 'ie') {
if(this.d){	
element.unselectable='on';
}
}
		if(!element)
			return false;

		if(!element.itemType)
			element = 	this.getElementToolbarItem(element);

		if(element.disabled)
			return;

		if(typeof(element.insertText) != 'undefined')
		{
			if(element.insertExtra)
			{
				insertCode = element.insertText+"_"+element.insertExtra;
			}
			else
			{
				insertCode = element.insertText;
			}

			if(this.openTags.indexOf(insertCode) != -1 || element.className.indexOf('editor_dropdown_menu_open') > -1)
			{
				this.setElementState(element, 'clicked');
			}
		}
		this.removeElementState(element, 'hover');
	},

	toolbarItemHover: function(e)
	{
		this.storeCaret();
		element = Event.element(e);
		if(MyBB.browser == 'ie') {
if(this.d){	
element.unselectable='on';
}
}
		if(!element)
			return false;

		if(!element.itemType)
			element = this.getElementToolbarItem(element);

		if(element.disabled)
			return;

		if(!element.className || element.className.indexOf('toolbar_clicked') == -1)
			this.setElementState(element, 'hover');
	},

	toolbarItemClick: function(e)
	{
		element = Event.element(e);
if(MyBB.browser == 'ie') {
if(this.d){	
element.unselectable='on';
}
}
		if(!element)
			return false;

		if(!element.itemType)
			element = this.getElementToolbarItem(element);

		if(element.disabled)
			return;

		if(element.dropdown && element.menu)
		{
			if(typeof(element.menu.activeItem) != "undefined")
			{
				Event.stop(e);
				if(!element.menu.lastItemValue)
				{
					this.showDropDownMenu(element.menu);
				}
				else
				{
				if(!this.d){
					this.insertMyCode(element.insertText, element.menu.lastItemValue, element.wysiwygText);
					}
				}

				return;
			}
		}

		if(element.id == "editor_item_close_tags")
		{
			this.closeTags();
		}
		else
		{
			if(typeof(element.insertExtra) != 'undefined')
				this.insertMyCode(element.insertText, element.insertExtra, element.wysiwygText);
			else
				this.insertMyCode(element.insertText,element.wysiwygText);
		}
	},

	insertList: function(type)
	{
		list = "";

		do
		{
			listItem = prompt(this.options.lang.enter_list_item, "");

			if(listItem != "" && listItem != null)
			{
				list = list+"[*]"+listItem+"\n";
			}
		}
		while(listItem != "" && listItem != null);

		if(list == "")
		{
			return false;
		}

		if(type)
		{
			list = "[list="+type+"]\n"+list;
		}
		else
		{
			list = "[list]\n"+list;
		}

		list = list+"[/list]\n";
		this.performInsert(list, "", true, false);
	},

	insertURL: function()
	{
	if(!this.d){
		selectedText = this.getSelectedText($(this.textarea));		
		url = prompt(this.options.lang.enter_url, "http://");

		if(url)
		{
			if(!selectedText)
			{
				title = prompt(this.options.lang.enter_url_title, "");
			}
			else
			{
				title = selectedText;
			}

			if(title)
			{
				this.performInsert("[url="+url+"]"+title+"[/url]", "", true, false);
			}
			else
			{
				this.performInsert("[url]"+url+"[/url]", "", true, false);
			}
		}
		}
		else{
		url = prompt(this.options.lang.enter_url, "http://");
		this.e.execCommand("createlink",0,url||null);
		}
	},

	insertEmail: function()
	{
		if(!this.d){
		selectedText = this.getSelectedText($(this.textarea));
		email = prompt(this.options.lang.enter_email, "");

		if(email)
		{
			if(!selectedText)
			{
				title = prompt(this.options.lang.enter_email_title, "");
			}
			else
			{
				title = selectedText;
			}

			if(title)
			{
				this.performInsert("[email="+email+"]"+title+"[/email]", "", true, false);
			}
			else
			{
				this.performInsert("[email]"+email+"[/email]", "", true, false);
			}
		}
				}else{
		email = prompt(this.options.lang.enter_email, "mailto:");
		this.e.execCommand("createlink",0,email||null);
		}
	},

	insertIMG: function()
	{
		image = prompt(this.options.lang.enter_image, "http://");
if(!this.d){
		if(image)
		{
			this.performInsert("[img]"+image+"[/img]", "", true);
		}
		
		}else{
		this.e.execCommand("insertimage",0,image||null);
		}
	},
	
	insertVideo: function(type)
	{
		selectedText = this.getSelectedText($(this.textarea));

		if(!selectedText)
		{
			url = prompt(this.options.lang.enter_video_url, "http://");
		}
		else
		{
			url = selectedText;
		}

		if(url)
		{
			this.performInsert("[video="+type+"]"+url+"[/video]", "", true, false);
		}
		this.setDropDownMenuActiveItem($('editor_item_video'), 0);	
	},
	resize: function ( uod ) {
	var sizeChangeStep = 50;
	var minimumHeight = 150;

	
	var msgEditorElement = document.getElementById("editor_toolbar_container_bottom").parentNode;
	var iframeAreaElement = document.getElementById("wysiwyg_"+this.textarea);
	var textAreaElement = document.getElementById(this.textarea);
	var AreaElement = document.getElementById("wysiwyg_"+this.textarea).parentNode;
	
	splitResult = msgEditorElement.style.height.split("px");
	newSize = parseInt( splitResult[0], 10 ) + uod * sizeChangeStep;
	if ( newSize > ( minimumHeight + 104 ) ) { msgEditorElement.style.height = newSize.toString(10) + "px"; } //
	if(MyBB.browser == 'ie') {
HeightEditor=minimumHeight+110;
}
else
{
HeightEditor=minimumHeight+70;
}
	if ( newSize > HeightEditor ) { iframeAreaElement.style.height = newSize.toString(10)-70 + "px"; 
textAreaElement.style.height = newSize.toString(10)-70 + "px";
AreaElement.style.height = newSize.toString(10)-70 + "px";  }
},
	insertMyCode: function(code, extra, wysiwygText)
	{
		this.restartEditorSelection();

		switch(code)
		{
			case "list":
					if(!this.d)
{
				this.insertList(extra);
				}
				break;
			case "resize1":
				this.resize(1);
				break;
			case "resize-1":
				this.resize(-1);
				break;
			case "url":
				this.insertURL();
				break;
			case "image":
				this.insertIMG();
				break;
			case "email":
				this.insertEmail();
				break;
			case "video":
				this.insertVideo(extra);
				break;
			case "unlink":
				break;
			case "removeformat":
				break;
			case "toggle":
				this.toggle();
				break;	
			default:
			if(!this.d){
				var already_open = false;
				var no_insert = false;
				if(extra)
				{
					var full_tag = code+"_"+extra;
				}
				else
				{
					var full_tag = code;
				}

				var newTags = new Array();
				this.openTags.each(function(tag)
				{
					exploded_tag = tag.split("_");
					if(exploded_tag[0] == code)
					{
						already_open = true;
						this.performInsert("[/"+exploded_tag[0]+"]", "", false);
						var elem = $('editor_item_'+exploded_tag[0]);

						if(elem)
						{
							this.removeElementState(elem, 'clicked');
						}

						if(elem && (elem.itemType == "dropdown" || elem.dropdown || elem.menu))
						{
							this.setDropDownMenuActiveItem(elem, 0);
						}

						if(tag == full_tag)
						{
							no_insert = true;
						}
					}
					else
					{
						newTags[newTags.length] = tag;
					}
				}.bind(this));

				this.openTags = newTags;
				var do_insert = false;

				if(extra != "" && extra != "-" && no_insert == false)
				{
					start_tag = "["+code+"="+extra+"]";
					end_tag = "[/"+code+"]";
					do_insert = true;
				}
				else if(!extra && already_open == false)
				{
					start_tag = "["+code+"]";
					end_tag = "[/"+code+"]";
					do_insert = true;
				}

				if(do_insert == true)
				{
					if(!this.performInsert(start_tag, end_tag, true))
					{
						this.openTags.push(full_tag);
						$('editor_item_close_tags').style.visibility = '';
					}
					else if($('editor_item_'+full_tag))
					{
						this.removeElementState($('editor_item_'+full_tag), 'clicked');
					}
					else if($('editor_item_'+code))
					{
						elem = $('editor_item_'+code);
						if(elem.type == "dropdown" || elem.dropdown || elem.menu)
							this.setDropDownMenuActiveItem($('editor_item_'+start_tag), 0);
					}
				}
		}

		if(this.openTags.length == 0)
		{
			$('editor_item_close_tags').style.visibility = 'hidden';
		}
		}
		if(this.d)
{
if(code=="code" || code=="quote" || code=="php"){
if(MyBB.browser == 'ie'){this.wysiwyg.contentWindow.focus();var c=this.e.selection.createRange().text;var starttxt="["+code+"]";var endtxt="[/"+code+"]";var txt=""+starttxt+""+c+""+endtxt+"";this.e.selection.createRange().text=txt;}else{this.wysiwyg.contentWindow.focus();this.wysiwyg.contentWindow.getSelection();var c=this.wysiwyg.contentWindow.getSelection();var range=c.getRangeAt(0);var clone=range.cloneContents(); c=new String(c); var txt="["+code+"]"+this.wysiwyg.contentWindow.getSelection()+"[/"+code+"]";this.e.execCommand('insertHTML',false,txt)}
	}else{
	 this.wysiwyg.contentWindow.focus();
	 
this.e.execCommand(wysiwygText, 0, extra);

	}
}	
	},

	getSelectedText: function(element)
	{
		element.focus();
		if(document.selection)
		{
			var selection = document.selection;
			var range = selection.createRange();

			if((selection.type == "Text" || selection.type == "None") && range != null)
			{
				return range.text;
			}
		}
		else if(element.selectionEnd)
		{
			var select_start = element.selectionStart;
			var select_end = element.selectionEnd;
			if(select_end <= 0)
			{
				select_end = element.textLength;
			}
			var start = element.value.substring(0, select_start);
			var middle = element.value.substring(select_start, select_end);
			return middle;
		}
	},

	performInsert: function(open_tag, close_tag, is_single, ignore_selection)
	{
		var is_closed = true;

		if(!ignore_selection)
		{
			var ignore_selection = false;
		}

		if(!close_tag)
		{
			var close_tag = "";
		}
		var textarea = $(this.textarea);
		textarea.focus();

		if(document.selection)
		{
			var selection = document.selection;
			var range = selection.createRange();

			if(ignore_selection != false)
			{
				selection.collapse;
			}

			if((selection.type == "Text" || selection.type == "None") && range != null && ignore_selection != true)
			{
				if(close_tag != "" && range.text.length > 0)
				{
					var keep_selected = true;
					range.text = open_tag+range.text+close_tag;
				}
				else
				{
					var keep_selected = false;

					if(is_single)
					{
						is_closed = false;
					}
					range.text = open_tag;
				}
				range.select();
			}
			else
			{
				textarea.value += open_tag;
			}
		}
		else if(typeof(textarea.selectionEnd) != 'undefined')
		{
			var select_start = textarea.selectionStart;
			var select_end = textarea.selectionEnd;
			var scroll_top = textarea.scrollTop;

			var start = textarea.value.substring(0, select_start);
			var middle = textarea.value.substring(select_start, select_end);
			var end = textarea.value.substring(select_end, textarea.textLength);

			if(select_end - select_start > 0 && ignore_selection != true && close_tag != "")
			{
				var keep_selected = true;
				middle = open_tag+middle+close_tag;
			}
			else
			{
				var keep_selected = false;
				if(is_single)
				{
					is_closed = false;
				}
				middle = open_tag;
			}

			textarea.value = start+middle+end;

			if(keep_selected == true && ignore_selection != true)
			{
				textarea.selectionStart = select_start;
				textarea.selectionEnd = select_start + middle.length;
			}
			else if(ignore_selection != true)
			{
				textarea.selectionStart = select_start + middle.length;
				textarea.selectionEnd = textarea.selectionStart;
			}
			textarea.scrollTop = scroll_top;
		}
		else
		{
			textarea.value += open_tag;

			if(is_single)
			{
				is_closed = false;
			}
		}
		this.updateOldArea();
		textarea.focus();
		this.trackingCaret = true;
		this.storeCaret();
		this.trackingCaret = false;		
		return is_closed;
	},

	closeTags: function()
	{
		if(this.openTags[0])
		{
			while(this.openTags[0])
			{
				tag = this.openTags.pop();
				exploded_tag = tag.split("_");
				this.performInsert("[/"+exploded_tag[0]+"]", "", false);

				if($('editor_item_'+exploded_tag[0]))
				{
					tag = $('editor_item_'+exploded_tag[0]);
				}
				else
				{
					tag = $('editor_item_'+tag);
				}
				if(tag)
				{
					if(tag.itemType == "dropdown" || tag.dropdown || tag.menu)
					{
						this.setDropDownMenuActiveItem(tag, 0);
					}
					else
					{
						this.removeElementState(tag, 'clicked');
					}
				}
			}
		}
		$(this.textarea).focus();
		$('editor_item_close_tags').style.visibility = 'hidden';
		this.openTags = new Array();
	},

	bindSmilieInserter: function(id)
	{
		if(!$(id))
		{
			return false;
		}

		var smilies = $(id).select('.smilie');

		if(smilies.length > 0)
		{
			smilies.each(function(smilie)
			{
				smilie.onclick = this.insertSmilie.bindAsEventListener(this);
				smilie.style.cursor = "pointer";
			}.bind(this));
		}
	},

	openGetMoreSmilies: function(editor)
	{
		MyBB.popupWindow('misc.php?action=smilies&popup=true&editor='+editor, 'sminsert', 240, 280);
	},

	insertSmilie: function(e)
	{
		element = Event.element(e);

		if(!element || !element.alt)
		{
			return false;
		}
		if(!this.d){
		this.performInsert(element.alt, "", true, false);
		}
		else{
		this.wysiwyg.contentWindow.focus();
		var txt='<img id="mybb_editor_smilie" src="'+element.src+'" alt="'+element.alt+'">';
		if(MyBB.browser == 'ie'){
		this.e.selection.createRange().pasteHTML(txt);
		}
		else{
		this.e.execCommand('insertHTML',0,txt);
		}
		}
	},

	insertAttachment: function(aid)
	{
	if(!this.d){
		this.performInsert("[attachment="+aid+"]", "", true, false);
	}
	else{
		this.wysiwyg.contentWindow.focus();
		var txt="[attachment="+aid+"]";
		if(MyBB.browser == 'ie'){
		this.e.selection.createRange().text=txt;
		}
		else{
		this.e.execCommand('insertHTML',0,txt);
		}
		}
	}
};





function ieHTML(b) {

		b = b.replace(/<P>&nbsp;<\/P>/gi, "<br>");	
		b = b.replace(/<p>(.*)<\/p>/gi, "$1<br>");
        b = b.replace(/<p(.*)>/gi, "<div$1>");
        b = b.replace(/<\/p(.*)>/gi, "</div$1>");
		return b
}









function HTMLtoBBcode(a) {
a=ieHTML(a);
            function r(re, str) {
                    a = a.replace(re, str);
            };
        if (isIE) {
            r(/<\/li>/gi, "");
            r(/<li>/gi, "[*]");
        }
        r(/<div><\/div>/gi, "");
        r(/<br[^>]*>/gi, "<br>");
		if(MyBB.browser !== 'ie'){
        r(/[\n\r]/gi, " ");
		}
		else
		{
		r(/[\r]/gi, " ");
		}
        r(/<script>(.*?)<\/script>/gi, "");
        r(/<script.*?>(.*?)<\/script>/gi, "");
        r(/<style>(.*?)<\/style>/gi, "");
        r(/<style.*?>(.*?)<\/style>/gi, "");
        r(/<w:.*?>(.*?)<\/w:.*?>/gi, "");
        r(/<o:p><\/o:p>/gi, "");
        r(/<img id="mybb_editor_smilie" src="(.*?)" alt="(.*?)".*?>/gi, "$2");		
		r(/<img id="mybb_editor_smilie" alt="(.*?)" src="(.*?)".*?>/gi, "$1");	
		r(/<img id=mybb_editor_smilie alt=(.*?) src="(.*?)".*?>/gi, "$1");	
        r(/<(abbr|acronym|applet|area|base|basefont|bdo|bgSound|big|body|button|caption|center|cite|code|col|colGroup|comment|custom|dd|del|dfn|dir|dl|dt|embed|fieldSet|frame|frameSet|head|html|ins|isIndex|kbd|label|legend|link|listing|map|marquee|menu|meta|noBR|noFrames|noScript|optGroup|option|param|plainText|pre|q|rt|ruby|samp|small|tBody|tFoot|tHead|title|tt|wbr|xml|xmp|th|script|form|input|iframe|object|select|textarea)(.*?)>/gi, "");
        r(/<\/(abbr|acronym|applet|area|base|basefont|bdo|bgSound|big|body|button|caption|center|cite|code|col|colGroup|comment|custom|dd|del|dfn|dir|dl|dt|embed|fieldSet|frame|frameSet|head|html|ins|isIndex|kbd|label|legend|link|listing|map|marquee|menu|meta|noBR|noFrames|noScript|optGroup|option|param|plainText|pre|q|rt|ruby|samp|small|tBody|tFoot|tHead|title|tt|wbr|xml|xmp|th|script|form|iframe|object|select|textarea)(.*?)>/gi, "");
        r(/\xA0/gi, " ");
        r(/<br[^>]*><\/div>/gi, "</div>");
        r(/<(ul|ol)><\/li>/gi, "<$1>");
        r(/<p([^>]*)>/gi, "<div$1>");
        r(/<\/p([^>]*)>/gi, "</div$1>\n");
        r(/\t/g, "     ");
        r(/\n /g, "\n");
		r(/\n/g, "");
		r(/<br[^>]*>/gi, "\n");
		r(/<hr[^>]*>/gi, "[hr]");
		r(/<a.*?href="mailto:(.*)".*?>(.*)<\/a>/gi,'[email=$1]$2[/email]');
        r(/<a.*?href=\"(.*?)\".*?>(.*?)<\/a>/gi, "[url=$1]$2[/url]");
        r(/<h([1-6])([^>]*)>/gi, "[B]");
        r(/<\/h([1-6])([^>]*)>/gi, "[/B]");
                r(/ (lang|start|class)=\"(.*?)\"/gi, "");
                r(/<span class="highlight">(.*?)<\/span>/gi, "$1");
                r(/&quot;/gi, "");              
        var b = a.split("<");
        var c = new Array;
        var e = 0;
        if (b.length > 1) {
            for (var i = 0; i < b.length; i++) {
                if (i > 0) {
                    b[i] = "<" + b[i];
                }
                var f = b[i];
                if (f.match(/<(div|span|font|strong|b|u|i|em|var|address|h1|h2|h3|h4|h5|h6|img|ol|ul|li|a|strike|s|sub|sup|hr|table|tr|td)( ([^>]{1,}.*?)){0,1}( {0,1}){0,1}>/i)) {
                    var g = RegExp.$1;
                    var h = RegExp.$3;
                    if (h.toLowerCase().indexOf("style=") != -1 && h.toLowerCase().indexOf("font-family:") != -1 && h.toLowerCase().indexOf("face=") != -1) {
                        h = h.replace(/ face="(.*?)"/gi, "");
                    } else if (h.toLowerCase().indexOf("style=") != -1 &&  h.toLowerCase().indexOf("color:") != -1 && h.toLowerCase().indexOf("color=") != -1) {
                        h = h.replace(/ color="(.*?)"/gi, "");
                    } else if (h.toLowerCase().indexOf("style=") != -1 &&  h.toLowerCase().indexOf("text-align:") != -1 && h.toLowerCase().indexOf("align=") != -1) {
                        h = h.replace(/ align="(.*?)"/gi, "");
                    }else if (h.toLowerCase().indexOf("style=") != -1 && h.toLowerCase().indexOf("align=") != -1) {
                        h = h.replace(/ align="(.*?)"/gi, "text-align:$1;");
                    }else if (h.toLowerCase().indexOf("type=") != -1) {
                        h = h.replace(/ type="(.*?)"/gi, "");
                    }
                    h = h.replace(/(color=|size=|face=|style=|align=)/gi, "|$1");
                    h = h.replace(/('|")/g, "");
                    h = h.replace(/ \|/g, "|");
                    var j = h.split("|");
                    var k = new Array;
                    if (j != null) {
                        for (var z = 0; z < j.length; z++) {
                            var l = j[z].split("=");
                            k[l[0].toLowerCase()] = j[z].replace(l[0].toLowerCase() + "=", "");
                        }
                    }
                    var m = "";
                    var g = g.toLowerCase();
                    if (g == "strike" || g == "s") {
                        if (k['style']) {
                            m = "[s]" + _process(g, k);
                        } else {
                            m = "[s]";
                        }
                    } else if (g == "sub") {
                        if (k['style']) {
                            m = "[sub]" + _process(g, k);
                        } else {
                            m = "[sub]";
                        }
                    } else if (g == "sup") {
                        if (k['style']) {
                            m = "[sup]" + _process(g, k);
                        } else {
                            m = "[sup]";
                        }
                    } else if (g == "li") {
                        if (k['style']) {
                            m = "[*]" + _process(g, k);
                        } else {
                            m = "[*]";
                        }
                    } else if (g == "strong" || g == "b") {
                        if (k['style']) {
                            if (k['style'].toLowerCase().indexOf("font-weight: bold") != -1 ||  k['style'].toLowerCase().indexOf("font-weight: 700") != -1) {
                                m = _process(g, k);
                            } else {
                                m = "[b]" + _process(g, k);
                            }
                        } else {
                            m = "[b]";
                        }
                    } else if (g == "em" ||
                        g == "i" || g == "var" || g == "address") {
                        if (k['style']) {
                            if (k['style'].toLowerCase().indexOf("font-style: italic") != -1) {
                                m = _process(g, k);
                            } else {
                                m = "[i]" + _process(g, k);
                            }
                        } else {
                            m = "[i]";
                        }
                    } else if (g == "u") {
                        if (k['style']) {
                            if (k['style'].toLowerCase().indexOf("text-decoration: underline") != -1) {
                                m = _process(g, k);
                            } else {
                                m = "[u]" + _process(g, k);
                            }
                        } else {
                            m = "[u]";
                        }
                    } else if (g == "ol") {
                        if (k['style']) {
                            m = _process(g, k);
                            if (m.indexOf("[list=a]") == -1) {
                                m += "[list=1]";
                            }
                        } else if (k.align) {
                            m = "[" + k.align.toUpperCase() + "]" + "[list=1]";
                        } else {
                            m = "[list=1]";
                        }
                    } else if (g == "ul") {
                        if (k['style']) {
                            m = _process(g, k) + "[list]";
                        } else if (k.align) {
                            m = "[" + k.align.toUpperCase() + "]" + "[list=1]";
                        } else {
                            m = "[list]";
                        }
                    } else if (g == "font" || g == "h1" || g == "h2" || g == "h3" || g == "h4" || g == "h5" || g == "h6") {
                        if (j.length > 0) {
                            for (var r in k) {
                                if (r == "color") {
                                    m += "[color=" + k.color + "]";
                                } else if (r == "size") {
                                    if (k.size) {
									if(k.size==0) m += "[size=xx-small]";
									else if(k.size==1) m += "[size=x-small]";
									else if(k.size==2) m += "[size=small]";
									else if(k.size==3) m += "[size=medium]";
									else if(k.size==4) m += "[size=large]";
									else if(k.size==5) m += "[size=x-large]";
									else if(k.size==6) m += "[size=xx-large]";
									else if(k.size>=7) m += "[size=xxx-large]";
									}
									
                                } else if (r == "face") {
                                    m += "[font=" + k.face + "]";
                                } else if (r == "style") {
                                    m += _process(g, k);
                                }
                            }
                        }
                    } else if (g == "div" || g == "span") {
                        if (k['style']) {
                            m = _process(g, k);
                        } else if (k.align) {
                            m = "[align=" + k.align.toUpperCase() + "]";
                        } else {
                            m = "[mybb]";
                        }
                    } else if (g == "img") {
                        if (isWebKit) {
                            f = f.replace(/<img(.*?)src="(.*?)">/gi, "[img]$2[/img]");
                        } else {
                            f.match(/<img(.*?)src="(.*?)"(.*?)>/gi);
                            var s = RegExp.$2;
                            s = s.replace("./", "");
                            if (s.toLowerCase().substr(0, 7) != "http://") {
                                var t = document.URL;
                                t = t.replace("http://", "");
                                var u = t.split("/");
                                var v = "http://";
                                for (var d = 0; d < u.length; d++) {
                                    if (d < u.length - 1) {
                                        v += u[d] + "/";
                                    }
                                }
                                f = f.replace(/\<img(.*?)src="(.*?)"(.*?)>/gi, "[img]" + v + s + "[/img]");
                            } else {
                                f = f.replace(/<img(.*?)src="(.*?)"(.*?)>/gi, "[img]$2[/img]");
                            }
                        }
                    }
                    b[i] = f.replace(/(<([^>]+)>)/, m);
                    if (g != "img") {
                        c[e] = m;
                        e++;

                    }
                } else if (f.match(/<\/(div|span|font|strong|b|u|i|em|var|address|h1|h2|h3|h4|h5|h6|ol|ul|li|a|strike|s|sub|sup|table|tr|td)>/i)) {
                    e--;
                    var w = c.pop();
                    if (w != null) {
                        var x = "";
                        var A = w;
                        A = A.replace(/=(.*?)\]/g, "]");
                        A = A.replace(/\]/g, "],");
                        A = A.replace(/\[(.*?)\]/g, "[/$1]");
                        var B = A.split(",");
                        B.reverse();
                        for (var y = 0; y < B.length; y++) {
                            x += B[y];
                        }
                        x = x.replace(/\[\/\*\]/gi, "");
                        b[i] = b[i].replace(/(<([^>]+)>)/, x);
                    } else {
                        b[i] = b[i].replace(/(<([^>]+)>)/, "");
                    }
                }
            }
            var C = b.join("");
        } else {
            var C = a;
        }
                function r2(re, str) {
                    C = C.replace(re, str);
            };
				r2(/\[font=.*?tahoma.*?\]/gi, "[font=tahoma]");		
        r2(/<[^>]*>/g, "");
        r2(/&lt;/g, "<");
        r2(/&gt;/g, ">");
        r2(/ /g, " ");
		r2(/&nbsp;/g, " ");
        r2(/&amp;/g, "&");
        r2(/     /g, "\t");
        r2(/\[mybb\]/g, "\n");
        r2(/\[mybb\]\n+/g, "\n");
        r2(/\[\/mybb\]\n+/g, "\n");
        if (isIE) {
                  r2(/\[\/mybb\]/g, "");
           }else{
                  r2(/\[\/mybb\]/g, "\n");
           }
        r2(/\[\*\]/gi, "\n[*]");
        r2(/\n\n\[\*\]/gi, "\n[*]");
        r2(/\[color=#.\w*\]\[\/color\]/gi, "");
                r2(/\[HIGHLIGHT=#.\w*\]\[\/HIGHLIGHT\]/gi,"");          
        r2(/\[size=\d\]\[\/size\]/gi, "");
        r2(/\[b\]\[\/b\]/gi, "");
        r2(/\[u\]\[\/u\]/gi, "");
        r2(/\[i\]\[\/i\]/gi, "");
        r2(/\[align=left\]\[\/align\]/gi, "");
        r2(/\[align=center\]\[\/align\]/gi, "");
        r2(/\[align=right\]\[\/align\]/gi, "");
        r2(/\[align=justify\]\[\/align\]/gi, "");
        r2(/\[url\]\[\/url\]/gi, "");
        r2(/\[s\]\[\/s\]/gi, "");
        r2(/\[sub\]\[\/sub\]/gi, "");
        r2(/\[sup\]\[\/sup\]/gi, "");
        r2(/\[img\]\[\/img\]/gi, "");
        r2(/^\n+/, "");
        r2(/\n+$/, "");
        return C;
}
function _process(a, b) {
	var c="";
	var d=b['style'].split(";");
	for(var j=0;j<d.length;j++){
		if(d[j]!=""&&d[j]!=null){
			var e=d[j].split(":");
			var f=e[0].toLowerCase().replace(/ /g,"");
			f=f.replace(/style=/gi,"");
			if(e[1]){
				var g=e[1].replace(/^ +| +$/g,"");
			}
                        if(f=="background-color")
                                {
                                        if(g.indexOf("#")==-1)
                                        {
                                                var h=this._toHex(g);
                                        } else {
                                                var h=g ;
                                        }
                                c+='[highlight='+h+']';
                                }
                                else if (f == "vertical-align" && g == "sub") {
                    c += "[sub]";
                } else if (f == "vertical-align" && g == "super") {
                    c += "[sup]";
                } else if (f == "list-style-type" && g == "lower-alpha") {
                    c += "[list=a]";
                } else if (f == "text-align") {
                    g = g.toUpperCase();
                    c += "[align=" + g + "]";
                } /*else if(f=="margin-left"||f=="margin-right"){
                                g=parseInt(g)/40;
                                for(var z=0;z<g;z++)
                                        {
                                        c+='[blockquote]';
                                        }
                        }*/else if (f == "font-weight") {
                    if (g.toUpperCase() == "BOLD" || g.toUpperCase() == "700") {
                        c += "[b]";
                    }
                } else if (f == "font-style") {
                    if (g.toUpperCase() == "ITALIC") {
                        c += "[i]";
                    }
                } else if (f == "font-family") {
                    c += "[font=" + g + "]";
                } else if (f == "font-size") {
					sizetype=g.substr((parseInt(g)+'').length,g.length);
					if(sizetype=="%") c +="[size="+parseInt(g)+"]";
					else if(sizetype=="pt" || sizetype!=="%" || sizetype!=="px" || sizetype!=="em" || sizetype!=="in"){
                    if (g == "7pt" || g == "xx-small") {
                        c += "[size=xx-small]";
                    } else if (g == "8pt" || g == "9pt" || g == "x-small") {
                        c += "[size=x-small]";
                    } else if (g == "10pt" || g == "11pt" || g == "small") {
                        c += "[size=small]";
                    } else if (g == "12pt" || g == "13pt" || g == "medium") {
                        c += "[size=medium]";
                    }   else if (g == "14pt" || g == "large") {
                        c += "[size=large]";
                    }   else if (g == "18pt" || g == "x-large") {
                        c += "[size=x-large]";
                    }   else if (g == "24pt" || g == "xx-large") {
                        c += "[size=xx-large]";
                    }   else if (parseInt(g) >= 14 && parseInt(g) < 18 || g == "large") {
                        c += "[size=large]";
                    } else if (parseInt(g) >= 18 && parseInt(g) < 24 || g == "x-large") {
                        c += "[size=x-large]";
                    } else if (parseInt(g) >= 24 && parseInt(g) < 36 || g == "xx-large") {
                        c += "[size=xx-large]";
                    } else if (parseInt(g) >= 36 || g == "-webkit-xxx-large") {
                        c += "[size=xxx-large]";
                    }
					}
                } else if (f == "text-decoration") {
                    if (g.toUpperCase() == "UNDERLINE") {
                        c += "[u]";
                    } else if (g.toUpperCase() == "LINE-THROUGH") {
                        c += "[s]";
                    }
                } else if (f == "color") {
                                        g=g.replace(/^ +| +$/g,"");
                    if (g.indexOf("#") == -1) {
                                        if (g.match(/rgb\((.*?)\)/gi)){
                                        var h = this._toHex(g);
                                        }else{
                       var h = g; }
                                           } else {
                        var h = g;
                    }
                    c += "[color=" + h + "]";
                }
            }
        }
        return c;
}
function _toHex(a)
        {
        a=a.replace(/rgb\((.*?)\)/gi,"$1");
        a=a.replace(/ /,"");
        var c=a.split(",");
        var r=parseInt(c[0]).toString(16);
        var g=parseInt(c[1]).toString(16);
        var b=parseInt(c[2]).toString(16);
        if(r.length==1)r="0"+r;
        if(g.length==1)g="0"+g;
        if(b.length==1)b="0"+b;
        return"#"+r+g+b
}
function BBcodetoHTML(a)
{
    function r(re, str) {
            a = a.replace(re, str);
    };
        r(/\n+(\[\/list\])/gi,'[/list]');
        r(/\[list\]\n+/gi,'[list]');
        r(/\[list=1\]\n+/gi,'[list=1]');
        r(/\[list=a\]\n+/gi,'[list=a]');
        r(/&/g,'&amp;');
        r(/</g,'&lt;');
        r(/>/g,'&gt;');
		r(/  /g,'&nbsp;&nbsp;');
		r(/\t/g,'&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;');
        r(/\n/g,'<br>');
		r(/\[i\](.*?)\[\/i\]/gi,'<i>$1</i>');
		r(/\[sub\](.*?)\[\/sub\]/gi,'<sub>$1</sub>');
		r(/\[sup\](.*?)\[\/sup\]/gi,'<sup>$1</sup>');
		r(/\[strike\](.*?)\[\/strike\]/gi,'<strike>$1</strike>');
		r(/\[b\](.*?)\[\/b\]/gi,'<b>$1</b>');
		r(/\[s\](.*?)\[\/s\]/gi,'<s>$1</s>');
		r(/\[u\](.*?)\[\/u\]/gi,'<u>$1</u>');
        r(/\[font=(.*?)\]/gi,'<font face="$1">');
        r(/\[color=(.*?)\]/gi,'<font color="$1">');
        r(/\[highlight=(.*?)\]/gi,'<span style="background-color:$1">');
        r(/\[\/highlight\]/gi,'</span>');      
        r(/\[size=(.*?)\]/gi,'<font style="font-size:$1">');
        r(/\[\/(font|color|size)\]/gi,'</font>');
        r(/\[align=(center|left|right|justify)\]/gi,'<div align="$1">');
        r(/\[\/align\]/gi,'</div>');
		r(/\[email=(.*?)\](.*?)\[\/email\]/gi,'<a href="mailto:$1">$2</a>');
        r(/\[url=(.*?)\]/gi,'<a href="$1">');
        r(/\[url\](.*?)\[\/url\]/gi,'<a href="$1">$1[/url]');
        r(/\[\/url\]/gi,'</a>');
        r(/\[img\](.*?)\[\/img\]/gi,'<img style="vertical-align:middle;" src="$1">');
        var b=a.match(/\[(list|list=1|list=a)\]/gi);
        r(/\[list=1\]/gi,'<ol>');
        r(/\[list=a\]/gi,'<ol style="list-style-type: lower-alpha">');
        r(/\[list\]/gi,'<ul>');
        r(/\[\*\]/gi,'<li>');
        r(/<br[^>]*><li>/gi,'<li>');
        r(/<br[^>]*> <li>/gi,'<li>');
        r(/<br[^>]*><\/li>/gi,'</li>');
		r(/\[hr\]/gi,'<hr>');
        if(b){for(var i=0;i<b.length;i++){if(b[i].toLowerCase()=="[list]"){r(/\[\/list\]/i,'</ul>');}else if(b[i].toLowerCase()=="[list=1]"||b[i].toLowerCase()=="[list=a]"){r(/\[\/list\]/i,'</ol>');}}}
        if(isOpera){r(/<\/table>/gi,'</tr></table>');r(/<\/tr>/gi,'</td></tr>');}      
        if(isOpera||isIE){r(/<li>/gi,'</li><li>');r(/<\/(ol|ul)>/gi,'</li></$1>');}
		a=smilies(a);
        return a;
}
var isIE;
var isOpera;
var isWebKit;
function smilies(a){var x=0 ;var regexp;if(typeof(mybbsmilies)=='undefined'){return a}for(x in mybbsmilies){regexp='/'+mybbsmilies[x][0]+'/gi';a=a.replace(eval(regexp),'<img id="mybb_editor_smilie" alt="'+mybbsmilies[x][1]+'" src="'+mybbsmilies[x][2]+'">')} return a}